@@ -12,10 +12,24 @@ from typing import (
     cast,
 )
 
-from ..const import InclusionStrategy, Protocols, QRCodeVersion, SecurityClass
+from ..const import (
+    MINIMUM_QR_STRING_LENGTH,
+    InclusionState,
+    InclusionStrategy,
+    Protocols,
+    QRCodeVersion,
+    RFRegion,
+    SecurityClass,
+    ZwaveFeature,
+)
+from .controller_statistics import ControllerStatistics, ControllerStatisticsDataType
 from ..event import Event, EventBase
 from .association import Association, AssociationGroup
 from .node import Node
+from ..util.helpers import (
+    convert_base64_to_bytes,
+    convert_bytes_to_base64,
+)
 
 if TYPE_CHECKING:
     from ..client import Client
@@ -56,7 +70,7 @@ class InclusionGrant:
 
 @dataclass
 class ProvisioningEntry:
-    """Representation of a provisioning entry."""
+    """Class to represent the base fields of a provisioning entry."""
 
     dsk: str
     security_classes: List[SecurityClass]
@@ -84,31 +98,19 @@ class ProvisioningEntry:
         )
 
 
-class QRProvisioningInformationDataType(TypedDict, total=False):
-    """Representation of provisioning information data dict type retrieved from a QR code."""
+@dataclass
+class NVMProgress:
+    """Class to represent an NVM backup/restore progress event."""
 
-    version: int  # required
-    securityClasses: List[int]  # required
-    dsk: str  # required
-    genericDeviceClass: int  # required
-    specificDeviceClass: int  # required
-    installerIconType: int  # required
-    manufacturerId: int  # required
-    productType: int  # required
-    productId: int  # required
-    applicationVersion: str  # required
-    maxInclusionRequestInterval: int
-    uuid: str
-    supportedProtocols: List[int]
+    bytes_read_or_written: int
+    total_bytes: int
 
 
 @dataclass
-class QRProvisioningInformation:
-    """Representation of provisioning information retrieved from a QR code."""
+class QRProvisioningInformationMixin:
+    """Mixin class to represent the base fields of a QR provisioning information."""
 
     version: QRCodeVersion
-    security_classes: List[SecurityClass]
-    dsk: str
     generic_device_class: int
     specific_device_class: int
     installer_icon_type: int
@@ -120,20 +122,26 @@ class QRProvisioningInformation:
     uuid: Optional[str]
     supported_protocols: Optional[List[Protocols]]
 
-    def to_dict(self) -> QRProvisioningInformationDataType:
-        """Return QRProvisioningInformationDataType dict from self."""
-        data = QRProvisioningInformationDataType(
-            version=self.version.value,
-            securityClasses=[sec_cls.value for sec_cls in self.security_classes],
-            dsk=self.dsk,
-            genericDeviceClass=self.generic_device_class,
-            specificDeviceClass=self.specific_device_class,
-            installerIconType=self.installer_icon_type,
-            manufacturerId=self.manufacturer_id,
-            productType=self.product_type,
-            productId=self.product_id,
-            applicationVersion=self.application_version,
-        )
+
+@dataclass
+class QRProvisioningInformation(ProvisioningEntry, QRProvisioningInformationMixin):
+    """Representation of provisioning information retrieved from a QR code."""
+
+    def to_dict(self) -> Dict[str, Any]:
+        """Return QRProvisioningInformation data dict from self."""
+        data = {
+            "version": self.version.value,
+            "securityClasses": [sec_cls.value for sec_cls in self.security_classes],
+            "dsk": self.dsk,
+            "genericDeviceClass": self.generic_device_class,
+            "specificDeviceClass": self.specific_device_class,
+            "installerIconType": self.installer_icon_type,
+            "manufacturerId": self.manufacturer_id,
+            "productType": self.product_type,
+            "productId": self.product_id,
+            "applicationVersion": self.application_version,
+            **(self.additional_properties or {}),
+        }
         if self.max_inclusion_request_interval is not None:
             data["maxInclusionRequestInterval"] = self.max_inclusion_request_interval
         if self.uuid is not None:
@@ -145,10 +153,8 @@ class QRProvisioningInformation:
         return data
 
     @classmethod
-    def from_dict(
-        cls, data: QRProvisioningInformationDataType
-    ) -> "QRProvisioningInformation":
-        """Return QRProvisioningInformation from QRProvisioningInformationDataType dict."""
+    def from_dict(cls, data: Dict[str, Any]) -> "QRProvisioningInformation":
+        """Return QRProvisioningInformation from data dict."""
         return cls(
             version=QRCodeVersion(data["version"]),
             security_classes=[
@@ -168,91 +174,29 @@ class QRProvisioningInformation:
                 Protocols(supported_protocol)
                 for supported_protocol in data.get("supportedProtocols", [])
             ],
+            additional_properties={
+                k: v
+                for k, v in data.items()
+                if k
+                not in {
+                    "version",
+                    "securityClasses",
+                    "dsk",
+                    "genericDeviceClass",
+                    "specificDeviceClass",
+                    "installerIconType",
+                    "manufacturerId",
+                    "productType",
+                    "productId",
+                    "applicationVersion",
+                    "maxInclusionRequestInterval",
+                    "uuid",
+                    "supportedProtocols",
+                }
+            },
         )
 
 
-class ControllerStatisticsDataType(TypedDict):
-    """Represent a controller statistics data dict type."""
-
-    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/zwave-js/src/lib/controller/ControllerStatistics.ts#L20-L39
-    messagesTX: int
-    messagesRX: int
-    messagesDroppedTX: int
-    messagesDroppedRX: int
-    NAK: int
-    CAN: int
-    timeoutACK: int
-    timeoutResponse: int
-    timeoutCallback: int
-
-
-class ControllerStatistics:
-    """Represent a controller statistics update."""
-
-    def __init__(self, data: Optional[ControllerStatisticsDataType] = None) -> None:
-        """Initialize controller statistics."""
-        self.data = data or ControllerStatisticsDataType(
-            CAN=0,
-            messagesDroppedRX=0,
-            messagesDroppedTX=0,
-            messagesRX=0,
-            messagesTX=0,
-            NAK=0,
-            timeoutACK=0,
-            timeoutCallback=0,
-            timeoutResponse=0,
-        )
-
-    @property
-    def messages_tx(self) -> int:
-        """Return number of messages successfully sent to controller."""
-        return self.data["messagesTX"]
-
-    @property
-    def messages_rx(self) -> int:
-        """Return number of messages received by controller."""
-        return self.data["messagesRX"]
-
-    @property
-    def messages_dropped_rx(self) -> int:
-        """Return number of messages from controller that were dropped by host."""
-        return self.data["messagesDroppedRX"]
-
-    @property
-    def messages_dropped_tx(self) -> int:
-        """
-        Return number of outgoing messages that were dropped.
-
-        These messages could not be sent.
-        """
-        return self.data["messagesDroppedTX"]
-
-    @property
-    def nak(self) -> int:
-        """Return number of messages that controller did not accept."""
-        return self.data["NAK"]
-
-    @property
-    def can(self) -> int:
-        """Return number of collisions while sending a message to controller."""
-        return self.data["CAN"]
-
-    @property
-    def timeout_ack(self) -> int:
-        """Return number of transmission attempts without an ACK from controller."""
-        return self.data["timeoutACK"]
-
-    @property
-    def timeout_response(self) -> int:
-        """Return number of transmission attempts where controller response timed out."""
-        return self.data["timeoutResponse"]
-
-    @property
-    def timeout_callback(self) -> int:
-        """Return number of transmission attempts where controller callback timed out."""
-        return self.data["timeoutCallback"]
-
-
 class ControllerDataType(TypedDict, total=False):
     """Represent a controller data dict type."""
 
@@ -275,6 +219,7 @@ class ControllerDataType(TypedDict, total=False):
     supportsTimers: bool
     isHealNetworkActive: bool
     statistics: ControllerStatisticsDataType
+    inclusionState: int
 
 
 class Controller(EventBase):
@@ -284,13 +229,12 @@ class Controller(EventBase):
         """Initialize controller."""
         super().__init__()
         self.client = client
-        self.data: ControllerDataType = state["controller"]
-        self._statistics = ControllerStatistics(self.data.get("statistics"))
         self.nodes: Dict[int, Node] = {}
         self._heal_network_progress: Optional[Dict[int, str]] = None
         for node_state in state["nodes"]:
             node = Node(client, node_state)
             self.nodes[node.node_id] = node
+        self.update(state["controller"])
 
     def __repr__(self) -> str:
         """Return the representation."""
@@ -406,6 +350,16 @@ class Controller(EventBase):
         """Return heal network progress state."""
         return self._heal_network_progress
 
+    @property
+    def inclusion_state(self) -> InclusionState:
+        """Return inclusion state."""
+        return InclusionState(self.data["inclusionState"])
+
+    def update(self, data: ControllerDataType) -> None:
+        """Update controller data."""
+        self.data = data
+        self._statistics = ControllerStatistics(self.data.get("statistics"))
+
     async def async_begin_inclusion(
         self,
         inclusion_strategy: Literal[
@@ -442,9 +396,26 @@ class Controller(EventBase):
             require_schema = 11
             # String is assumed to be the QR code string so we can pass as is
             if isinstance(provisioning, str):
+                if len(
+                    provisioning
+                ) < MINIMUM_QR_STRING_LENGTH or not provisioning.startswith("90"):
+                    raise ValueError(
+                        f"QR code string must be at least {MINIMUM_QR_STRING_LENGTH} characters "
+                        "long and start with `90`"
+                    )
                 options["provisioning"] = provisioning
+            # If we get a Smart Start QR code, we provision the node and return because
+            # inclusion is over
+            elif (
+                isinstance(provisioning, QRProvisioningInformation)
+                and provisioning.version == QRCodeVersion.SMART_START
+            ):
+                raise ValueError(
+                    "Smart Start QR codes can't use the normal inclusion process. Use the "
+                    "provision_smart_start_node command to provision this device."
+                )
             # Otherwise we assume the data is ProvisioningEntry or
-            # QRProvisioningInformation
+            # QRProvisioningInformation that is not a Smart Start QR code
             else:
                 options["provisioning"] = provisioning.to_dict()
 
@@ -462,6 +433,13 @@ class Controller(EventBase):
         provisioning_info: Union[ProvisioningEntry, QRProvisioningInformation, str],
     ) -> None:
         """Send provisionSmartStartNode command to Controller."""
+        if (
+            isinstance(provisioning_info, QRProvisioningInformation)
+            and provisioning_info.version == QRCodeVersion.S2
+        ):
+            raise ValueError(
+                "An S2 QR Code can't be used to pre-provision a Smart Start node"
+            )
         await self.client.async_send_command(
             {
                 "command": "controller.provision_smart_start_node",
@@ -573,11 +551,19 @@ class Controller(EventBase):
             require_schema = 11
             # String is assumed to be the QR code string so we can pass as is
             if isinstance(provisioning, str):
+                if len(
+                    provisioning
+                ) < MINIMUM_QR_STRING_LENGTH or not provisioning.startswith("90"):
+                    raise ValueError(
+                        f"QR code string must be at least {MINIMUM_QR_STRING_LENGTH} characters "
+                        "long and start with `90`"
+                    )
                 options["provisioning"] = provisioning
             # Otherwise we assume the data is ProvisioningEntry or
             # QRProvisioningInformation
             else:
                 options["provisioning"] = provisioning.to_dict()
+
         data = await self.client.async_send_command(
             {
                 "command": "controller.replace_failed_node",
@@ -751,6 +737,85 @@ class Controller(EventBase):
             }
         )
 
+    async def async_supports_feature(self, feature: ZwaveFeature) -> Optional[bool]:
+        """
+        Send supportsFeature command to Controller.
+
+        When None is returned it means the driver does not yet know whether the
+        controller supports the input feature.
+        """
+        data = await self.client.async_send_command(
+            {"command": "controller.supports_feature", "feature": feature.value},
+            require_schema=12,
+        )
+        return cast(Optional[bool], data.get("supported"))
+
+    async def async_get_state(self) -> None:
+        """Get controller state."""
+        data = await self.client.async_send_command(
+            {"command": "controller.get_state"}, require_schema=14
+        )
+        self.update(data["state"])
+
+    async def async_backup_nvm_raw(self) -> bytes:
+        """Send backupNVMRaw command to Controller."""
+        data = await self.client.async_send_command(
+            {"command": "controller.backup_nvm_raw"}, require_schema=14
+        )
+        return convert_base64_to_bytes(data["nvmData"])
+
+    async def async_restore_nvm(self, file: bytes) -> None:
+        """Send restoreNVM command to Controller."""
+        await self.client.async_send_command(
+            {
+                "command": "controller.restore_nvm",
+                "nvmData": convert_bytes_to_base64(file),
+            },
+            require_schema=14,
+        )
+
+    async def async_get_power_level(self) -> Dict[str, int]:
+        """Send getPowerlevel command to Controller."""
+        data = await self.client.async_send_command(
+            {"command": "controller.get_powerlevel"}, require_schema=14
+        )
+        return {
+            "power_level": data["powerlevel"],
+            "measured_0_dbm": data["measured0dBm"],
+        }
+
+    async def async_set_power_level(
+        self, power_level: int, measured_0_dbm: int
+    ) -> bool:
+        """Send setPowerlevel command to Controller."""
+        data = await self.client.async_send_command(
+            {
+                "command": "controller.set_powerlevel",
+                "powerlevel": power_level,
+                "measured0dBm": measured_0_dbm,
+            },
+            require_schema=14,
+        )
+        return cast(bool, data["success"])
+
+    async def async_get_rf_region(self) -> RFRegion:
+        """Send getRFRegion command to Controller."""
+        data = await self.client.async_send_command(
+            {"command": "controller.get_rf_region"}, require_schema=14
+        )
+        return RFRegion(data["region"])
+
+    async def async_set_rf_region(self, rf_region: RFRegion) -> bool:
+        """Send setRFRegion command to Controller."""
+        data = await self.client.async_send_command(
+            {
+                "command": "controller.set_rf_region",
+                "region": rf_region.value,
+            },
+            require_schema=14,
+        )
+        return cast(bool, data["success"])
+
     def receive_event(self, event: Event) -> None:
         """Receive an event."""
         if event.data["source"] == "node":
@@ -824,3 +889,21 @@ class Controller(EventBase):
 
     def handle_validate_dsk_and_enter_pin(self, event: Event) -> None:
         """Process a validate dsk and enter pin event."""
+
+    def handle_nvm_backup_progress(self, event: Event) -> None:
+        """Process a nvm backup progress event."""
+        event.data["nvm_backup_progress"] = NVMProgress(
+            event.data["bytesRead"], event.data["total"]
+        )
+
+    def handle_nvm_convert_progress(self, event: Event) -> None:
+        """Process a nvm convert progress event."""
+        event.data["nvm_convert_progress"] = NVMProgress(
+            event.data["bytesRead"], event.data["total"]
+        )
+
+    def handle_nvm_restore_progress(self, event: Event) -> None:
+        """Process a nvm restore progress event."""
+        event.data["nvm_restore_progress"] = NVMProgress(
+            event.data["bytesWritten"], event.data["total"]
+        )
