@@ -1,8 +1,14 @@
 """Provide a model for the Z-Wave JS node."""
 from typing import TYPE_CHECKING, Any, Dict, List, Optional, TypedDict, Union, cast
 
-from ..const import INTERVIEW_FAILED, CommandClass, NodeStatus, SecurityClass
-from ..event import Event
+from ..const import (
+    INTERVIEW_FAILED,
+    CommandClass,
+    NodeStatus,
+    PowerLevel,
+    SecurityClass,
+)
+from ..event import Event, EventBase
 from ..exceptions import (
     FailedCommand,
     NotFoundError,
@@ -10,6 +16,7 @@ from ..exceptions import (
     UnwriteableValue,
 )
 from .command_class import CommandClassInfo, CommandClassInfoDataType
+from .device_class import DeviceClass, DeviceClassDataType
 from .device_config import DeviceConfig, DeviceConfigDataType
 from .endpoint import Endpoint, EndpointDataType
 from .firmware import (
@@ -18,6 +25,13 @@ from .firmware import (
     FirmwareUpdateProgress,
     FirmwareUpdateProgressDataType,
 )
+from .node_health_check import (
+    CheckHealthProgress,
+    LifelineHealthCheckSummary,
+    RouteHealthCheckSummary,
+    TestPowerLevelProgress,
+)
+from .node_statistics import NodeStatistics, NodeStatisticsDataType
 from .notification import (
     EntryControlNotification,
     EntryControlNotificationDataType,
@@ -39,67 +53,14 @@ if TYPE_CHECKING:
     from ..client import Client
 
 
-class NodeStatisticsDataType(TypedDict):
-    """Represent a node statistics data dict type."""
-
-    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/zwave-js/src/lib/node/NodeStatistics.ts#L21-L33
-    commandsTX: int
-    commandsRX: int
-    commandsDroppedTX: int
-    commandsDroppedRX: int
-    timeoutResponse: int
-
-
-class NodeStatistics:
-    """Represent a node statistics update."""
-
-    def __init__(self, data: Optional[NodeStatisticsDataType] = None) -> None:
-        """Initialize node statistics."""
-        self.data = data or NodeStatisticsDataType(
-            commandsDroppedRX=0,
-            commandsDroppedTX=0,
-            commandsRX=0,
-            commandsTX=0,
-            timeoutResponse=0,
-        )
-
-    @property
-    def commands_tx(self) -> int:
-        """Return number of commands successfully sent to node."""
-        return self.data["commandsTX"]
-
-    @property
-    def commands_rx(self) -> int:
-        """
-        Return number of commands received from node.
-
-        Includes responses to sent commands.
-        """
-        return self.data["commandsRX"]
-
-    @property
-    def commands_dropped_rx(self) -> int:
-        """Return number of commands from node that were dropped by host."""
-        return self.data["commandsDroppedRX"]
-
-    @property
-    def commands_dropped_tx(self) -> int:
-        """
-        Return number of outgoing commands that were dropped.
-
-        These commands could not be sent.
-        """
-        return self.data["commandsDroppedTX"]
-
-    @property
-    def timeout_response(self) -> int:
-        """Return number of Get-type cmds where node's response didn't come in time."""
-        return self.data["timeoutResponse"]
-
-
-class NodeDataType(EndpointDataType):
+class NodeDataType(TypedDict, total=False):
     """Represent a node data dict type."""
 
+    nodeId: int  # required
+    index: int  # required
+    deviceClass: DeviceClassDataType  # required
+    installerIcon: int
+    userIcon: int
     name: str
     location: str
     status: int  # 0-4  # required
@@ -134,39 +95,24 @@ class NodeDataType(EndpointDataType):
     commandClasses: List[CommandClassInfoDataType]
     values: List[ValueDataType]
     statistics: NodeStatisticsDataType
+    highestSecurityClass: int
+    isControllerNode: bool
 
 
-class Node(Endpoint):
+class Node(EventBase):
     """Represent a Z-Wave JS node."""
 
     def __init__(self, client: "Client", data: NodeDataType) -> None:
         """Initialize the node."""
-        super().__init__(client, data)
-        self.data: NodeDataType = data
-        self._device_config = DeviceConfig(self.data.get("deviceConfig", {}))
-        self._statistics = NodeStatistics(self.data.get("statistics"))
+        super().__init__()
+        self.client = client
+        self.data: NodeDataType = {}
+        self._device_config: DeviceConfig = DeviceConfig({})
+        self._statistics: NodeStatistics = NodeStatistics()
         self._firmware_update_progress: Optional[FirmwareUpdateProgress] = None
-        self.values: Dict[str, Union[Value, ConfigurationValue]] = {}
-        for val in data["values"]:
-            value_id = _get_value_id_from_dict(self, val)
-            try:
-                self.values[value_id] = _init_value(self, val)
-            except UnparseableValue:
-                # If we can't parse the value, don't store it
-                pass
-
-        self.endpoints = {
-            endpoint["index"]: Endpoint(
-                self.client,
-                endpoint,
-                {
-                    value_id: value
-                    for value_id, value in self.values.items()
-                    if self.index == value.endpoint
-                },
-            )
-            for endpoint in self.data["endpoints"]
-        }
+        self.values: Dict[str, Union[ConfigurationValue, Value]] = {}
+        self.endpoints: Dict[int, Endpoint] = {}
+        self.update(data)
 
     def __repr__(self) -> str:
         """Return the representation."""
@@ -184,6 +130,31 @@ class Node(Endpoint):
             self.client.driver == other.client.driver and self.node_id == other.node_id
         )
 
+    @property
+    def node_id(self) -> int:
+        """Return node ID property."""
+        return self.data["nodeId"]
+
+    @property
+    def index(self) -> int:
+        """Return index property."""
+        return self.data["index"]
+
+    @property
+    def device_class(self) -> DeviceClass:
+        """Return the device_class."""
+        return DeviceClass(self.data["deviceClass"])
+
+    @property
+    def installer_icon(self) -> Optional[int]:
+        """Return installer icon property."""
+        return self.data.get("installerIcon")
+
+    @property
+    def user_icon(self) -> Optional[int]:
+        """Return user icon property."""
+        return self.data.get("userIcon")
+
     @property
     def status(self) -> NodeStatus:
         """Return the status."""
@@ -349,6 +320,72 @@ class Node(Endpoint):
         """Return firmware update progress."""
         return self._firmware_update_progress
 
+    @property
+    def highest_security_class(self) -> Optional[SecurityClass]:
+        """Return highest security class configured on the node."""
+        if (security_class := self.data.get("highestSecurityClass")) is None:
+            return None
+        return SecurityClass(security_class)
+
+    @property
+    def is_controller_node(self) -> bool:
+        """Return whether the node is a controller node."""
+        return self.data["isControllerNode"]
+
+    @property
+    def keep_awake(self) -> bool:
+        """Return whether the node is set to keep awake."""
+        return self.data["keepAwake"]
+
+    def update(self, data: NodeDataType) -> None:
+        """Update the internal state data."""
+        self.data = data
+        self._device_config = DeviceConfig(self.data.get("deviceConfig", {}))
+        self._statistics = NodeStatistics(self.data.get("statistics"))
+
+        # Remove stale values
+        value_ids = (_get_value_id_from_dict(self, val) for val in data["values"])
+        self.values = {
+            value_id: val
+            for value_id, val in self.values.items()
+            if value_id in value_ids
+        }
+
+        # Populate new values
+        for val in data["values"]:
+            try:
+                if (value_id := _get_value_id_from_dict(self, val)) in self.values:
+                    self.values[value_id].update(val)
+                else:
+                    self.values[value_id] = _init_value(self, val)
+            except UnparseableValue:
+                # If we can't parse the value, don't store it
+                pass
+
+        # Remove stale endpoints
+        self.endpoints = {
+            idx: endpoint
+            for idx, endpoint in self.endpoints.items()
+            if idx in (endpoint["index"] for endpoint in self.data["endpoints"])
+        }
+
+        # Add new endpoints or update existing ones
+        for endpoint in self.data["endpoints"]:
+            idx = endpoint["index"]
+            values = {
+                value_id: value
+                for value_id, value in self.values.items()
+                if self.index == value.endpoint
+            }
+            if idx in self.endpoints:
+                self.endpoints[idx].update(endpoint, values)
+            else:
+                self.endpoints[idx] = Endpoint(
+                    self.client,
+                    endpoint,
+                    values,
+                )
+
     def get_command_class_values(
         self, command_class: CommandClass, endpoint: int = None
     ) -> Dict[str, Union[ConfigurationValue, Value]]:
@@ -384,8 +421,8 @@ class Node(Endpoint):
         """
         Send a node command. For internal use only.
 
-        If wait_for_result is not None, it will take precedence, otherwise we will decide to wait
-        or not based on the node status.
+        If wait_for_result is not None, it will take precedence, otherwise we will decide
+        to wait or not based on the node status.
         """
         kwargs = {}
         message = {"command": f"node.{cmd}", "nodeId": self.node_id, **cmd_kwargs}
@@ -512,6 +549,22 @@ class Node(Endpoint):
         )
         return cast(bool, data.get("responded", False))
 
+    async def async_invoke_cc_api(
+        self,
+        command_class: CommandClass,
+        method_name: str,
+        *args: Any,
+        wait_for_result: Optional[bool] = None,
+    ) -> Any:
+        """Call endpoint.invoke_cc_api command."""
+        return await self.endpoints[0].async_invoke_cc_api(
+            command_class, method_name, *args, wait_for_result=wait_for_result
+        )
+
+    async def async_supports_cc_api(self, command_class: CommandClass) -> bool:
+        """Call endpoint.supports_cc_api command."""
+        return await self.endpoints[0].async_supports_cc_api(command_class)
+
     async def async_has_security_class(self, security_class: SecurityClass) -> bool:
         """Return whether node has the given security class."""
         data = await self.async_send_command(
@@ -531,6 +584,126 @@ class Node(Endpoint):
         assert data
         return SecurityClass(data["highestSecurityClass"])
 
+    async def async_test_power_level(
+        self, test_node_id: int, power_level: PowerLevel, test_frame_count: int
+    ) -> int:
+        """Send testPowerLevel command to Node."""
+        data = await self.async_send_command(
+            "test_powerlevel",
+            testNodeId=test_node_id,
+            powerlevel=power_level,
+            testFrameCount=test_frame_count,
+            require_schema=13,
+            wait_for_result=True,
+        )
+        assert data
+        return cast(int, data["framesAcked"])
+
+    async def async_check_lifeline_health(
+        self, rounds: Optional[int] = None
+    ) -> LifelineHealthCheckSummary:
+        """Send checkLifelineHealth command to Node."""
+        kwargs = {}
+        if rounds is not None:
+            kwargs["rounds"] = rounds
+        data = await self.async_send_command(
+            "check_lifeline_health",
+            require_schema=13,
+            wait_for_result=True,
+            **kwargs,
+        )
+        assert data
+        return LifelineHealthCheckSummary(data["summary"])
+
+    async def async_check_route_health(
+        self, target_node_id: int, rounds: Optional[int] = None
+    ) -> RouteHealthCheckSummary:
+        """Send checkRouteHealth command to Node."""
+        kwargs = {"targetNodeId": target_node_id}
+        if rounds is not None:
+            kwargs["rounds"] = rounds
+        data = await self.async_send_command(
+            "check_route_health",
+            require_schema=13,
+            wait_for_result=True,
+            **kwargs,
+        )
+        assert data
+        return RouteHealthCheckSummary(data["summary"])
+
+    async def async_get_state(self) -> None:
+        """Get node state."""
+        data = await self.async_send_command(
+            "get_state", require_schema=14, wait_for_result=True
+        )
+        assert data
+        self.update(data["state"])
+
+    async def async_set_name(
+        self, name: str, update_cc: bool = True, wait_for_result: Optional[bool] = None
+    ) -> None:
+        """Set node name."""
+        # If we may not potentially update the name CC, we should just wait for the
+        # result because the change is local to the driver
+        if not update_cc:
+            wait_for_result = True
+        await self.async_send_command(
+            "set_name",
+            name=name,
+            updateCC=update_cc,
+            wait_for_result=wait_for_result,
+            require_schema=14,
+        )
+        self.data["name"] = name
+
+    async def async_set_location(
+        self,
+        location: str,
+        update_cc: bool = True,
+        wait_for_result: Optional[bool] = None,
+    ) -> None:
+        """Set node location."""
+        # If we may not potentially update the location CC, we should just wait for the
+        # result because the change is local to the driver
+        if not update_cc:
+            wait_for_result = True
+        await self.async_send_command(
+            "set_location",
+            location=location,
+            updateCC=update_cc,
+            wait_for_result=wait_for_result,
+            require_schema=14,
+        )
+        self.data["location"] = location
+
+    async def async_set_keep_awake(self, keep_awake: bool) -> None:
+        """Set node keep awake state."""
+        await self.async_send_command(
+            "set_keep_awake",
+            keepAwake=keep_awake,
+            wait_for_result=True,
+            require_schema=14,
+        )
+        self.data["keepAwake"] = keep_awake
+
+    def handle_test_powerlevel_progress(self, event: Event) -> None:
+        """Process a test power level progress event."""
+        event.data["test_power_level_progress"] = TestPowerLevelProgress(
+            event.data["acknowledged"], event.data["total"]
+        )
+
+    def handle_check_lifeline_health_progress(self, event: Event) -> None:
+        """Process a check lifeline health progress event."""
+        event.data["check_lifeline_health_progress"] = CheckHealthProgress(
+            event.data["rounds"], event.data["totalRounds"], event.data["lastRating"]
+        )
+
+    def handle_check_route_health_progress(self, event: Event) -> None:
+        """Process a check route health progress event."""
+        event.data["check_route_health_progress"] = CheckHealthProgress(
+            event.data["rounds"], event.data["totalRounds"], event.data["lastRating"]
+        )
+
     def handle_wake_up(self, event: Event) -> None:
         """Process a node wake up event."""
         # pylint: disable=unused-argument
@@ -574,18 +747,7 @@ class Node(Endpoint):
     def handle_ready(self, event: Event) -> None:
         """Process a node ready event."""
         # the event contains a full dump of the node
-        self.data.update(event.data["nodeState"])
-        # update device config
-        if new_device_config := self.data.get("deviceConfig"):
-            self._device_config = DeviceConfig(new_device_config)
-        # update/add values
-        for value_state in event.data["nodeState"]["values"]:
-            value_id = _get_value_id_from_dict(self, value_state)
-            value = self.values.get(value_id)
-            if value is None:
-                self.values[value_id] = _init_value(self, value_state)
-            else:
-                value.update(value_state)
+        self.update(event.data["nodeState"])
 
     def handle_value_added(self, event: Event) -> None:
         """Process a node value added event."""
